"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFiles = exports.File = void 0;
const path_1 = __importDefault(require("path"));
const globby_1 = __importDefault(require("globby"));
const lodash_1 = require("lodash");
class File {
    constructor({ cwd, fileName, source, migration, }) {
        this.cwd = cwd;
        this.fileName = fileName;
        this._source = source;
        this._stats = undefined;
        this.migration = migration;
    }
    get path() {
        // TODO: support ablsolute paths as well
        return path_1.default.join(this.cwd, this.fileName);
    }
    get stats() {
        if (!this._stats) {
            try {
                this._stats = this.migration.fs.lstatSync(this.path);
            }
            catch (err) {
                this._stats = null;
            }
        }
        return this._stats;
    }
    get exists() {
        // this.stats is null in case the file isn't exists
        return !!this.stats;
    }
    get isDirectory() {
        var _a;
        return !!((_a = this.stats) === null || _a === void 0 ? void 0 : _a.isDirectory());
    }
    get source() {
        if (lodash_1.isUndefined(this._source)) {
            this._source = this.migration.fs.readFileSync(this.path);
        }
        return this._source;
    }
}
exports.File = File;
const getFiles = (cwd, pattern, migration) => {
    const fileNames = globby_1.default.sync(pattern, {
        cwd,
        gitignore: true,
        ignore: ['**/node_modules/**', '**/.git/**'],
        dot: true,
        // @ts-expect-error
        fs: migration.fs.fileSystemAdapterMethods,
    });
    if (!fileNames) {
        return [];
    }
    const files = fileNames.map((fileName) => {
        return new File({ cwd, fileName, migration });
    });
    return files;
};
exports.getFiles = getFiles;
