/// <reference types="node" />
import { BigIntOptions, Dirent, PathLike, Stats } from 'fs-extra';
import { FileSystemAdapter } from 'fast-glob';
declare type FileInformation = {
    data?: string;
    stats?: Stats;
    action: 'remove' | 'write' | 'none';
};
export declare class VirtualFileSystem implements FileSystemAdapter {
    cwd: string;
    map: Map<string, FileInformation>;
    constructor({ cwd }: {
        cwd: string;
    });
    private getFile;
    statSync(path: PathLike, options: BigIntOptions): Stats;
    readdirSync(path: PathLike, options: any): Array<Dirent | DirentLike>;
    getWrittenFilePaths(): string[];
    getRemovedFilePaths(): string[];
    writeFileSync(path: string, data: string): void;
    removeSync(path: string): void;
    lstatSync(path: PathLike): Stats;
    readFileSync(path: string): string;
    renameSync(oldPath: string, newPath: string): void;
    existsSync(path: string): boolean;
    ensureFileSync(path: string): void;
    writeChangesToDisc(): void;
    fileSystemAdapterMethods: {
        readdirSync: (path: PathLike, options: any) => Array<Dirent | DirentLike>;
        lstatSync: (path: PathLike) => Stats;
        statSync: (path: PathLike, options: BigIntOptions) => Stats;
    };
}
declare class DirentLike implements Dirent {
    name: string;
    constructor(name: string);
    isFile: () => boolean;
    isDirectory: () => boolean;
    isBlockDevice: () => boolean;
    isCharacterDevice: () => boolean;
    isSymbolicLink: () => boolean;
    isFIFO: () => boolean;
    isSocket: () => boolean;
}
export {};
