"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCli = void 0;
const arg_1 = __importDefault(require("arg"));
const path_1 = __importDefault(require("path"));
const chalk_1 = __importDefault(require("chalk"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const runMigration_1 = require("../runMigration");
/**
 *
 * @param options.binName the name of the bin file, as presented in the --help output
 * @param options.migrationFile absolute path to the migrationFile
 */
const createCli = async ({ binName, migrationFile: customMigrationFile, version, }) => {
    process.on('unhandledRejection', (error) => {
        throw error;
    });
    const args = arg_1.default({
        // Types
        '--version': Boolean,
        '--help': Boolean,
        '--verbose': Boolean,
        '--cwd': String,
        '--dry': Boolean,
        '--yes': Boolean,
        '--quiet': Boolean,
        // Aliases
        '-v': '--version',
        '-h': '--help',
        '-d': '--dry',
        '-y': '--yes',
        '-q': '--quiet',
    }, {
        permissive: false,
    });
    if (args['--version']) {
        console.log(version);
        process.exit(0);
    }
    const migrationFile = customMigrationFile || args._[0];
    if (!migrationFile && args['--help']) {
        console.log(`
        Usage
          $ ${binName}${migrationFile ? '' : ' <path/to/migration.ts>'}

        Options
          --version, -v   Version number
          --help, -h      Displays this message
          --dry, -d       Dry-run mode, does not modify files
          --yes, -y       Skip all confirmation prompts. Useful in CI to automatically answer the confirmation prompt
          --cwd           Runs the migration on this directory [defaults to process.cwd()]
          --quiet, -q     Runs on quiet mode (does not print results)
      `);
        process.exit(0);
    }
    let migrationFileAbsolutePath;
    if (!path_1.default.isAbsolute(migrationFile)) {
        migrationFileAbsolutePath = path_1.default.join(process.cwd(), migrationFile);
    }
    else {
        migrationFileAbsolutePath = migrationFile;
    }
    if (!fs_extra_1.default.existsSync(migrationFileAbsolutePath)) {
        console.error(chalk_1.default.red(`couldn't find a migration file on "${migrationFileAbsolutePath}"`));
        process.exit(1);
    }
    const customCwd = args['--cwd'];
    const cwdAbsolutePath = customCwd && path_1.default.join(process.cwd(), customCwd);
    if (cwdAbsolutePath && !fs_extra_1.default.existsSync(cwdAbsolutePath)) {
        console.error(chalk_1.default.red(`passed cwd "${cwdAbsolutePath}" does not exists`));
        process.exit(1);
    }
    const cwd = cwdAbsolutePath || process.cwd();
    await runMigration_1.runMigration({
        cwd,
        migrationFilePath: migrationFileAbsolutePath,
        dry: !!args['--dry'],
        yes: !!args['--yes'],
        quiet: !!args['--quiet'],
    });
};
exports.createCli = createCli;
