"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createReport = exports.formatSingleTaskError = exports.formatSingleTaskResult = void 0;
const chalk_1 = require("chalk");
const lodash_1 = require("lodash");
const ERROR_TEXT = 'ERROR';
const PASS_TEXT = 'PASS';
const ERROR = chalk_1.supportsColor
    ? chalk_1.reset.inverse.bold.red(` ${ERROR_TEXT} `)
    : ERROR_TEXT;
const PASS = chalk_1.supportsColor
    ? chalk_1.reset.inverse.bold.green(` ${PASS_TEXT} `)
    : PASS_TEXT;
const formatSingleTaskResult = (taskResult) => {
    switch (taskResult.type) {
        case 'transform': {
            return `${PASS} ${taskResult.newFile.fileName}`;
        }
        case 'create': {
            return `${PASS} ${taskResult.newFile.fileName}`;
        }
        case 'remove': {
            return `${PASS} ${taskResult.file.fileName}`;
        }
        case 'rename': {
            const { originalFile, newFile } = taskResult;
            return `${PASS} ${originalFile.fileName} -> ${newFile.fileName}`;
        }
        default: {
            // @ts-expect-error ts thinks that task is "never"
            throw new Error(`unknown taskResult type "${taskResult.type}"`);
        }
    }
};
exports.formatSingleTaskResult = formatSingleTaskResult;
const formatSingleTaskError = (taskError) => {
    var _a;
    return `${ERROR} ${(_a = taskError.file) === null || _a === void 0 ? void 0 : _a.fileName}

${taskError.error.stack}`;
};
exports.formatSingleTaskError = formatSingleTaskError;
const createReport = (migration) => {
    const output = [];
    for (const [taskTitle, taskResults] of Object.entries(lodash_1.groupBy(migration.results, 'task.title'))) {
        output.push(chalk_1.underline(chalk_1.bold(taskTitle)) +
            '\n' +
            taskResults.map(exports.formatSingleTaskResult).join('\n'));
    }
    if (migration.errors.length > 0) {
        output.push('⚠️  ' +
            chalk_1.red(`The following migration tasks were failed, but you can still migrate the rest` +
                ' ⚠️'));
        for (const [taskTitle, taskErrors] of Object.entries(lodash_1.groupBy(migration.errors, 'task.title'))) {
            output.push(chalk_1.underline(chalk_1.bold(taskTitle)) +
                '\n' +
                taskErrors.map(exports.formatSingleTaskError).join('\n'));
        }
    }
    if (lodash_1.isEmpty(migration.results)) {
        output.push(chalk_1.blue('🤷‍ No changes have been made'));
    }
    return output.join('\n\n');
};
exports.createReport = createReport;
