"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.runMigration = void 0;
const prompts_1 = __importDefault(require("prompts"));
const Migration_1 = require("./Migration");
const loadUserMigrationFile_1 = require("./loadUserMigrationFile");
const lodash_1 = require("lodash");
/**
 *
 * @param options.cwd The directory of the project which the migration runs on
 * @param options.migrationFilePath path to the migration file
 * @param options.dry dry run mode
 * @param options.yes do not prompt the user with confirmation
 * Run a migration
 * @param options.quiet runs on quiet mode (does not print the result)
 *
 */
const runMigration = async ({ cwd, migrationFilePath, dry, yes, quiet, }) => {
    const migration = Migration_1.Migration.init({ cwd, quiet });
    const { events } = migration;
    await loadUserMigrationFile_1.loadUserMigrationFile(migration, migrationFilePath);
    events.emit('migration-after-run', { migration, options: { dry } });
    if (dry) {
        process.exit(0);
    }
    if (lodash_1.isEmpty(migration.results)) {
        process.exit(0);
    }
    if (!yes) {
        events.emit('migration-before-prompt');
        const response = await prompts_1.default({
            type: 'confirm',
            name: 'value',
            message: `Press 'y' to execute the migration on the above files`,
            initial: true,
        });
        if (!response.value) {
            events.emit('migration-after-prompt-aborted');
            process.exit(1);
        }
        events.emit('migration-after-prompt-confirmed');
    }
    migration.write();
    events.emit('migration-after-write');
};
exports.runMigration = runMigration;
