"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runCreateTask = void 0;
const File_1 = require("../File");
const lodash_1 = require("lodash");
const runCreateTask = (task, migration) => {
    let taskResults = [];
    let taskErrors = [];
    let createdFiles = [];
    if (!task.pattern) {
        try {
            // @ts-expect-error
            const createdFile = task.fn();
            createdFiles.push(createdFile);
        }
        catch (error) {
            const taskError = {
                type: task.type,
                task,
                error,
            };
            migration.events.emit('task-fail', taskError);
            taskErrors.push(taskError);
        }
    }
    else {
        const files = File_1.getFiles(migration.options.cwd, task.pattern, migration);
        for (let file of files) {
            migration.events.emit('task-start', { file, task });
            try {
                const createdFile = task.fn({
                    fileName: file.fileName,
                    source: file.source,
                });
                createdFiles.push(createdFile);
            }
            catch (error) {
                const taskError = {
                    type: task.type,
                    task,
                    error,
                };
                migration.events.emit('task-fail', taskError);
                taskErrors.push(taskError);
            }
        }
    }
    for (let createdFile of createdFiles) {
        if (lodash_1.isNull(createdFile)) {
            migration.events.emit('create-success-abort', { task });
            continue;
        }
        if (!createdFile.fileName) {
            throw new Error('the return value of a create function needs to contain an object with { fileName: <string> }');
        }
        if (lodash_1.isUndefined(createdFile.source)) {
            throw new Error('the return value of a create function needs to contain an object with { source: <string> }');
        }
        const originalFile = new File_1.File({
            cwd: migration.options.cwd,
            fileName: createdFile.fileName,
            migration,
        });
        const newFile = new File_1.File({
            cwd: migration.options.cwd,
            fileName: createdFile.fileName,
            source: createdFile.source,
            migration,
        });
        const taskResult = {
            newFile,
            type: task.type,
            task,
        };
        if (originalFile.exists) {
            // Add the original File to mark that the file exists
            taskResult.originalFile = originalFile;
            // @ts-expect-error - we know that originalFile exists here
            migration.events.emit('create-success-override', taskResult);
        }
        migration.events.emit('task-success', taskResult);
        migration.fs.writeFileSync(newFile.path, newFile.source);
        taskResults.push(taskResult);
    }
    return { taskErrors, taskResults };
};
exports.runCreateTask = runCreateTask;
