"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runRenameTask = void 0;
const lodash_1 = require("lodash");
const File_1 = require("../File");
const runRenameTask = (task, migration) => {
    const files = File_1.getFiles(migration.options.cwd, task.pattern, migration);
    let taskResults = [];
    let taskErrors = [];
    for (let file of files) {
        migration.events.emit('task-start', { file, task });
        let renamedFile;
        try {
            renamedFile = task.fn(file);
        }
        catch (error) {
            const taskError = {
                type: task.type,
                file,
                error,
                task,
            };
            migration.events.emit('task-fail', taskError);
            taskErrors.push(taskError);
            continue;
        }
        const newFile = new File_1.File({
            cwd: migration.options.cwd,
            fileName: renamedFile || file.fileName,
            source: file.source,
            migration,
        });
        const isRenamed = !lodash_1.isEqual(newFile.fileName, file.fileName);
        if (isRenamed) {
            const taskResult = {
                task,
                originalFile: file,
                newFile,
                type: task.type,
            };
            migration.events.emit('task-success', taskResult);
            migration.fs.renameSync(file.path, newFile.path);
            taskResults.push(taskResult);
            continue;
        }
        migration.events.emit('task-noop', {
            task,
            file,
        });
    }
    return { taskResults, taskErrors };
};
exports.runRenameTask = runRenameTask;
