"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runTransformTask = void 0;
const lodash_1 = require("lodash");
const File_1 = require("../File");
const utils_1 = require("../utils");
const runTransformTask = (task, migration) => {
    const files = File_1.getFiles(migration.options.cwd, task.pattern, migration);
    let aborted = false;
    const abort = () => {
        aborted = true;
    };
    let taskResults = [];
    let taskErrors = [];
    for (let file of files) {
        // We let the user control this callback
        // If it was call, we want to stop looping the files
        if (aborted) {
            break;
        }
        migration.events.emit('task-start', { file, task });
        let transformedSource;
        try {
            transformedSource = task.fn({
                source: file.source,
                fileName: file.fileName,
                abort,
            });
        }
        catch (error) {
            const taskError = {
                file,
                type: task.type,
                task,
                error,
            };
            migration.events.emit('task-fail', taskError);
            taskErrors.push(taskError);
            continue;
        }
        let source;
        if (lodash_1.isObject(transformedSource)) {
            source = utils_1.strigifyJson(transformedSource);
        }
        else {
            source = transformedSource;
        }
        const newFile = new File_1.File({
            cwd: migration.options.cwd,
            fileName: file.fileName,
            source,
            migration,
        });
        const isRenamed = !lodash_1.isEqual(newFile.fileName, file.fileName);
        const isModified = isRenamed || !lodash_1.isEqual(newFile.source, file.source);
        if (isModified) {
            const taskResult = {
                originalFile: file,
                newFile,
                type: task.type,
                task,
            };
            migration.events.emit('task-success', taskResult);
            taskResults.push(taskResult);
            continue;
        }
        migration.events.emit('task-noop', {
            task,
            file,
        });
    }
    if (aborted) {
        return { taskErrors: [], taskResults: [] };
    }
    taskResults.forEach((result) => {
        if (result.type !== 'transform') {
            throw new Error('wrong action type');
        }
        migration.fs.removeSync(result.originalFile.path);
        migration.fs.writeFileSync(result.newFile.path, result.newFile.source);
    });
    return { taskResults, taskErrors };
};
exports.runTransformTask = runTransformTask;
